@@ -41,13 +41,12 @@ import subprocess
 import functools
 import pathlib
 import multiprocessing.connection
-import concurrent.futures
 from contextlib import contextmanager
 
 import msgpack
 import pylibmc
 
-VERSION = "0.6.9"
+VERSION = "0.7.0"
 LOCK_SH = 0
 LOCK_EX = 1
 RUN_PATH = "/run/starrydb/"
@@ -199,7 +198,7 @@ class StarryServer(object):
         self.address = address
         self.cursor = 0
         self.depots = {}
-        self.logger = StarryLog(log_file, eval(f'logging.{log_level}'))
+        self.logger = StarryLog(log_file, getattr(logging, log_level, logging.WARNING))
         self.rw_ratio = rw_ratio
         self.memcached = None
 
@@ -253,7 +252,7 @@ class StarryServer(object):
             except:
                 pass
 
-    def start(self, max_workers=512, backlog=512, authkey=b'starry'):
+    def start(self, backlog=1024, authkey=b'starry'):
         if pathlib.Path(str(self.address)).is_socket():
             try:
                 os.remove(self.address)
@@ -261,36 +260,35 @@ class StarryServer(object):
                 pass
 
         with multiprocessing.connection.Listener(self.address, backlog=backlog, authkey=authkey) as listener:
-            with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
-                while True:
-                    try:
-                        conn = listener.accept()
-                    except ConnectionError as e:
-                        self.logger.warn(f"Caught exception: {e.__doc__}({e})")
-                        continue
-                    except OSError as e:
-                        self.logger.warn(f"Caught exception: {e.__doc__}({e})")
-                        break
-                    except Exception as e:
-                        self.logger.warn(f"Caught exception: {e.__doc__}({e})")
-                        continue
+            while True:
+                try:
+                    conn = listener.accept()
+                except ConnectionError as e:
+                    self.logger.warn(f"Caught exception: {e.__doc__}({e})")
+                    continue
+                except OSError as e:
+                    self.logger.warn(f"Caught exception: {e.__doc__}({e})")
+                    break
+                except Exception as e:
+                    self.logger.warn(f"Caught exception: {e.__doc__}({e})")
+                    continue
+
+                try:
+                    (depot, mode) = conn.recv()
+
+                    if depot not in self.depots:
+                        self.depots[depot] = StarryDepot(depot, self.logger, self.rw_ratio, self.cursor, self.memcached)
+
+                        self.cursor += 1
+
+                    threading.Thread(target=self.depots[depot].executeCommand, args=(conn, mode)).start()
+                except Exception as e:
+                    self.logger.error(f"Caught exception: {e.__doc__}({e})")
 
                     try:
-                        (depot, mode) = conn.recv()
-
-                        if depot not in self.depots:
-                            self.depots[depot] = StarryDepot(depot, self.logger, self.rw_ratio, self.cursor, self.memcached)
-
-                            self.cursor += 1
-
-                        executor.submit(self.depots[depot].executeCommand, conn, mode)
-                    except Exception as e:
-                        self.logger.error(f"Caught exception: {e.__doc__}({e})")
-
-                        try:
-                            conn.close()
-                        except:
-                            pass
+                        conn.close()
+                    except:
+                        pass
 
 class StarryDepot(object):
     def __init__(self, path, logger, rw_ratio, index, memcached):
@@ -321,7 +319,7 @@ class StarryDepot(object):
                     break
 
                 try:
-                    conn.send(eval(f"self.{func}")(*param))
+                    conn.send(getattr(self, func)(*param))
                 except Exception as e:
                     self.logger.debug(f"Caught exception: {e.__doc__}({e})")
 
@@ -340,7 +338,7 @@ class StarryDepot(object):
                         break
 
                     try:
-                        conn.send(eval(f"self.{func}")(*param))
+                        conn.send(getattr(self, func)(*param))
                     except Exception as e:
                         self.logger.debug(f"Caught exception: {e.__doc__}({e})")
 
@@ -494,7 +492,7 @@ class StarryDepot(object):
             return (True, values)
 
     def updateMulti(self, values):
-        mark = {"mode": "w", "members": list(values.keys())}
+        mark = {"mode": "w", "members": list(values)}
 
         with self._lock(mark):
             for key, value in values.items():
@@ -505,7 +503,7 @@ class StarryDepot(object):
             return (True, "")
 
     def insertMulti(self, values):
-        mark = {"mode": "w", "members": list(values.keys())}
+        mark = {"mode": "w", "members": list(values)}
 
         with self._lock(mark):
             for key, value in values.items():
@@ -1234,7 +1232,7 @@ def _getPart(value, branch, filters):
                     return value
 
             for v in value:
-                for key in list(v.keys()):
+                for key in list(v):
                     if key not in filters:
                         v.pop(key)
 
@@ -1244,7 +1242,7 @@ def _getPart(value, branch, filters):
                 if key not in value:
                     return value
 
-            for key in list(value.keys()):
+            for key in list(value):
                 if key not in filters:
                     value.pop(key)
 
